<?php
/**
 * Doing a simple search
 *
 * Searching all data base tables with notes and characters and so on.
 *
 *
 * @author Andreas Plank <andreas.plank@web.de>
 * @version 2009-03-29 17:32:42
 * @license Noncommercial-Share Alike 3.0 Unported
 * @category program tools
 * @package chirokey_larvae
 * @subpackage program
 * @todo refine search, bugs?
 * @todo adapt search to specimen search
 */
/**
 * Extract current server path relatively
 *
 *
 * {1} means one level up as it replaces
 * any string exept '/' multiple times ('+') ended by '/'. In regular expression
 * language:
 * <code>([^/]+/)</code>
 * The (..) can be used to refere the found string by \\1, \\2 and so on.
 * @global string $currentServerPath
 */
$currentServerPath = preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__);
include($currentServerPath.'structure/session.inc.php');
include($currentServerPath.'structure/functions.php');
include($currentServerPath.'structure/classes/sql.php');//first get classes
include($currentServerPath.'structure/config.php');/* login */
include($currentServerPath.'structure/classes/tools.php');
header('content-type: text/html; charset=utf-8');
?>
<html>
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
  <title>Searching<?php echo empty($_POST['words'])?"":": ".preg_replace("# +#",", ",$_POST['words']); ?></title>
  <link rel="stylesheet" type="text/css" href="../structure/css/basic.css">
  <script type='text/javascript' src='../structure/js/functions.js'></script>
  <script type='text/javascript' src='../structure/js/overlib.js'></script>
</head>
<body >
<div id="overDiv" style="position:absolute; visibility:hidden; z-index:1000;"></div>
<!--  page content  -->
<div class="content">
<h1>Simple search for notes/taxa</h1>
<div class="info" style="font-size:smaller;">
  You can use regular expressions as explained as follows, but a search of [a-z]+ won't find special characters like &aelig;. But [[:alpha:]]+ should do the work.
  <ul>
    <li><span class="highlight">chironomid[aes]+</span> which searches for chironomid and additionally 1 or more characters of 'aes' and can result in: chironomids, chironomidae, Chironomids or Chironomidae. Note the order of characters between [...] is irrelevantly.</li>
    <li><span class="highlight">S[&amp;aelig;æ]+ther</span> for: Saether, Sæther and the HTML code variant S&amp;aelig;ther</li>
    <li><span class="highlight">S[&amp;aelig;æ]+ther (19|20)[0-9]{2}</span> is the same as above but additionally with a number with 4 digits in the given range: 2 digits with either 19 or 20 and then 3rd &amp; 4th digit: a range between 0-9 which will find for instance 'Sæther 1969' or 'Sæther 2005'</li>
    <li><span class="highlight">S[&amp;aelig;æ]+ther +and [a-zA-Z;&]+ +(19|20)[0-9]{2}</span> same as above but additionally it finds for instance: Sæther and Wang 1993 or any other '... and Author DDDD' combination whereas D stands here for the digits. The <span class="highlight">&nbsp;+</span> takes optionally multiple spaces.</li>
    <li>to search for 'Rossaro et al. 2004' or 'Rossaro and Delettre 1992' you can use <span class="highlight">Rossaro +(et al.|and +[a-zA-Z;&amp;]+) (19|20)[0-9]{2}</span></li>
  </ul>
</div>
<?php
//  print_r($_POST);
//  print_r($_POST);
$objMysql = new db_mysql;//class db_mysql
$objHtml = new html;//class html
# @annotation: doSearch isset?
if(isset($_POST['doSearch']) && !empty($_POST['words'])){
//   $strSearched=htmlentities($_POST['words'],ENT_NOQUOTES,'UTF-8');
  $strSql=preg_replace(
      array(// search
        "# +#", #1 remove multiple spaces
        "#'#", #2 spaces-end
      ),
      array(// replace
        " ",  #1 remove multiple spaces
        "\'",   #2 spaces-end
      ),
      $_POST['words']
    );
  # @annotation: doSearch: MySQL different tables
    switch ($_POST['table']) {
      case "characters": # @annotation: MySQL characters ($strInfo, $strSql)
      $strInfo = "
          SELECT
            `characters`.`id` AS 'id_characters',
            `characters`.`notes` ,
            `characters`.`ecology`,
            `characters`.`names_valid_id` AS 'genus_valid_id',
            IFNULL(`names_authors`.`authors_short`, 'no special author') AS 'author',
            CONCAT( `names_genera`.`name`, ' ',`names_species`.`name` ) AS 'taxon',
            CONCAT(
              IF(STRCMP(IFNULL(`names_valid`.`in_brackets`,''),1),'','('),
              `tempb`.`authors_short`,
              IF(STRCMP(IFNULL(`names_valid`.`in_brackets`,''),1),'',')')
            ) AS 'descriptor'
          FROM (`names_genera`, `characters`,
            (
              SELECT
                CONCAT(`authors_short`,`literal`) AS 'authors_short',
                `id`
                FROM `names_authors`
            ) AS `tempb`
          )
          LEFT JOIN `names_authors`   ON `characters`.`names_authors_id`   = `names_authors`.`id`
          LEFT JOIN `names_valid` ON `names_genera`.`id` = `names_valid`.`genus_name_id`
          LEFT JOIN `names_species` ON `names_valid`.`species_name_id` = `names_species`.`id`
          -- LEFT JOIN `tempb` ON `names_valid`.`descriptor_name_id`  = `tempb`.`id`
          -- LEFT JOIN `characters` ON `names_valid`.`id` = `characters`.`names_valid_id`
          -- LEFT JOIN `names_valid` ON `characters`.`names_valid_id` = `names_valid`.`id`
          WHERE (
            (
              (`characters`.`ecology` REGEXP  '".$strSql."') OR
              (`characters`.`notes` REGEXP  '".$strSql."') OR
              (`names_genera`.`name` REGEXP  '".$strSql."') OR
              (`names_species`.`name` REGEXP  '".$strSql."')
            )
            AND
            `tempb`.`id` =  `names_valid`.`descriptor_name_id` AND
            `names_valid`.`id` = `characters`.`names_valid_id`
          )
          ORDER by 'taxon' ASC
          ;";
          break;
      case "images":# @annotation: MySQL images ($strInfo, $strSql)
       $strInfo = "SELECT STRAIGHT_JOIN
          `images`.`id` AS 'image_id',
          `images`.`filename`,
          `images`.`annote`,
          `images`.`time_added`,
          IFNULL(CONCAT('modified: ',`images`.`time_modified`),'') AS 'time_modified',
            CONCAT(
              IF(STRCMP(IFNULL(`names_valid`.`in_brackets`,''),1),'','('),
              `tempb`.`authors_short`,
              IF(STRCMP(IFNULL(`names_valid`.`in_brackets`,''),1),'',')')
            )  AS 'descriptor',
          `tempb`.`id` AS 'descriptor_id',
          CONCAT(
            IFNULL(`names_genera`.`name`,''),' ',
            IFNULL(CONCAT('(',`names_subgenus`.`name`,')'),''),' ',
            `names_species`.`name`
          ) AS 'taxon',
          `morph_attrib`.`morph_long`,
          CONCAT(`names_authors`.`authors_short`,`names_authors`.`literal`) AS 'authors_short',
          `names_authors`.`id` AS 'authors_id',
          `names_authors`.`title` AS 'title',
          `names_authors`.`journal` AS 'journal',
          `images`.`names_valid_id`
        FROM(
          `names_subfamily`,
          `morph_attrib`,
          `images`,
          `names_authors`,
          `names_valid`,
          `names_species`,
          `names_genera`,
            (
              SELECT
                CONCAT(`authors_short`,`literal`) AS 'authors_short',
                `id`
                FROM `names_authors`
            ) AS `tempb`
          )
        LEFT JOIN `names_subgenus` ON `names_valid`.`subgenus_name_id`=`names_subgenus`.`id`
        WHERE
          `tempb`.`id` =  `names_valid`.`descriptor_name_id` AND
          `images`.`morph_id`               = `morph_attrib`.`id` AND
          `names_genera`.`id`               = `names_valid`.`genus_name_id` AND
          `names_species`.`id`              = `names_valid`.`species_name_id` AND
          (
            `images`.`annote` REGEXP BINARY '".$strSql."' OR
            `names_authors`.`authors_short` REGEXP BINARY '".$strSql."' OR
            `names_authors`.`title`  REGEXP BINARY '".$strSql."' OR
            `names_species`.`name`  REGEXP BINARY '".$strSql."' OR
            `names_genera`.`name`   REGEXP BINARY '".$strSql."'
          ) AND
          `names_valid`.`id`                = `images`.`names_valid_id` AND
          `names_valid`.`subfamily_name_id` = `names_subfamily`.`id` AND
          `names_authors`.`id`              = `images`.`names_authors_id`
        ORDER BY 'taxon', 'authors_short', 'morph_long' ASC;";
      break;
    case "synonyms": # @annotation: MySQL synonyms ($strInfo, $addMySQL, $strSql)
      $addMySQL = " AND `synonyms`.`note` REGEXP BINARY '".$strSql."'";
      include($currentServerPath.'structure/config.php');/* variables */
      $strInfo = $MysqlSelect['Synonyms'];
      break;
    case "names_valid": # @annotation: MySQL names_valid ($strInfo, $strSql)
    $strInfo = "SELECT
        CONCAT(
          IFNULL(`names_genera`.`name`,''),' ',
          IFNULL(CONCAT('(',`names_subgenus`.`name`,')'),''),' ',
          `names_species`.`name`
        ) AS 'taxon',
        `names_subfamily`.`name` AS 'subfamily',
        `names_valid`.`id` AS 'valid_id',
        `names_valid`.`notes`,
        CONCAT(
            IF(STRCMP(IFNULL(`names_valid`.`in_brackets`,''),1),'','('),
              `names_authors`.`authors_short`,
            IF(STRCMP(IFNULL(`names_valid`.`in_brackets`,''),1),'',')')
            ) AS 'valid_author',
        `names_authors`.`id` AS 'valid_author_id'
      FROM `names_subfamily`
      LEFT JOIN `names_valid`    ON `names_subfamily`.`id` = `names_valid`.`subfamily_name_id`
      LEFT JOIN `names_authors`  ON `names_valid`.`descriptor_name_id` = `names_authors`.`id`
      LEFT JOIN `names_genera`   ON `names_valid`.`genus_name_id` = `names_genera`.`id`
      LEFT JOIN `names_species`  ON `names_valid`.`species_name_id` = `names_species`.`id`
      LEFT JOIN `names_subgenus` ON `names_valid`.`subgenus_name_id` = `names_subgenus`.`id`
      WHERE
        `names_valid`.`notes`  REGEXP BINARY '".$strSql."' OR
        `names_genera`.`name`  REGEXP BINARY '".$strSql."' OR
        `names_species`.`name` REGEXP BINARY '".$strSql."'
        ORDER by 'taxon' ASC
      ;
      ";
      break;
    case "references": # @annotation: MySQL references ($strInfo, $strSql)
    $strInfo = "
    SELECT *
        FROM `names_authors`
        WHERE
        `author_names` REGEXP BINARY '".$strSql."'
        OR `authors_short` REGEXP BINARY '".$strSql."'
        OR `editor_name` REGEXP BINARY '".$strSql."'
        OR `title` REGEXP BINARY '".$strSql."'
        OR `journal` REGEXP BINARY '".$strSql."'
        OR `journal_url` REGEXP BINARY '".$strSql."'
        OR `abstract` REGEXP BINARY '".$strSql."'
        OR `note` REGEXP BINARY '".$strSql."'
        OR `keywords` REGEXP BINARY '".$strSql."'
      ORDER BY 'author_names', 'literal', 'title' ASC
      ;
      ";
      break;
}


  $objResult=$objMysql->queryObjectArray($strInfo);

}//end update description (notes & ecology)
?>
<form method="POST" action="">

  <input type="text" name="words" value="<?php echo preg_replace('#&([a-z]+;)#','&amp;\\1', $_POST['words']);/* HTML entities */?>">
  <input type="submit" value="Go" name="doSearch"><br>
  <label >
    <input type="radio" value="characters" name="table" <?php echo isset($_POST['table'])?($_POST['table']=='characters'?'checked':''):'checked';?>>
   ... in table characters
  </label><br>
  <label >
    <input type="radio" value="images" name="table" <?php echo $_POST['table']=='images'?'checked':'';?>>
    ... in table images
  </label><br>
  <label >
    <input type="radio" value="synonyms" name="table" <?php echo $_POST['table']=='synonyms'?'checked':'';?>>
    ... in table synonyms
  </label><br>
  <label >
    <input type="radio" value="names_valid" name="table" <?php echo $_POST['table']=='names_valid'?'checked':'';?>>
    ... in table names_valid
  </label><br>
  <label >
    <input type="radio" value="references" name="table" <?php echo $_POST['table']=='references'?'checked':'';?>>
    ... in table references
  </label>
  <?

  ?>
</form>
<?
if(!$objResult && isset($_POST['doSearch']))
  print_r("<div class='info'>No results".(empty($_POST['words'])? '. Please insert a string.':' for "'.preg_replace('#&([a-z]+;)#','&amp;\\1', $_POST['words']).'"')."</div>");
    elseif(isset($_POST['doSearch'])) {
    foreach($objResult as $valueFound){
      #print_r($valueFound);
      switch ($_POST['table']) {
        case "characters": # @annotation: $_POST[doSearch] characters
          $arrFound['text'][$valueFound->id_characters] =strip_tags($valueFound->ecology." ".$valueFound->notes);
          $arrFound['taxon'][$valueFound->id_characters]=$valueFound->taxon;
          $arrFound['id_characters'][$valueFound->id_characters]=$valueFound->id_characters;
          $arrFound['genus_valid_id'][$valueFound->id_characters]=$valueFound->genus_valid_id;
          $arrFound['descriptor'][$valueFound->id_characters]=$valueFound->descriptor;
          $arrFound['author'][$valueFound->id_characters]=$valueFound->author;
          break;
        case "images": # @annotation: $_POST[doSearch] images
          $arrFound['image_id']      [$valueFound->image_id]=$valueFound->image_id;
          $arrFound['filename']      [$valueFound->image_id]=$valueFound->filename;
          $arrFound['text']          [$valueFound->image_id]=$valueFound->annote;
          $arrFound['time_added']    [$valueFound->image_id]=$valueFound->time_added;
          $arrFound['time_modified'] [$valueFound->image_id]=$valueFound->time_modified;
          $arrFound['descriptor']    [$valueFound->image_id]=$valueFound->descriptor;
          $arrFound['descriptor_id'] [$valueFound->image_id]=$valueFound->descriptor_id;
          $arrFound['taxon']         [$valueFound->image_id]=$valueFound->taxon;
          $arrFound['morph_long']    [$valueFound->image_id]=$valueFound->morph_long;
          $arrFound['authors_short'] [$valueFound->image_id]=$valueFound->authors_short;
          $arrFound['authors_id']    [$valueFound->image_id]=$valueFound->authors_id;
          $arrFound['title']         [$valueFound->image_id]=$valueFound->title;
          $arrFound['journal']       [$valueFound->image_id]=$valueFound->journal;
          $arrFound['names_valid_id'][$valueFound->image_id]=$valueFound->names_valid_id;
          break;
        case "synonyms":# @annotation: $_POST[doSearch] synonyms
          $arrFound['text']               [$valueFound->synonym_id] = $valueFound->note;
          $arrFound['synonym_author']     [$valueFound->synonym_id] = $valueFound->synonym_author;
          $arrFound['synonym_author_id']  [$valueFound->synonym_id] = $valueFound->synonym_author_id;
          $arrFound['synonym_id']         [$valueFound->synonym_id] = $valueFound->synonym_id;
          $arrFound['synonym_name']       [$valueFound->synonym_id] = $valueFound->synonym_name;
          $arrFound['valid_author']       [$valueFound->synonym_id] = $valueFound->valid_author;
          $arrFound['valid_author_id']    [$valueFound->synonym_id] = $valueFound->valid_author_id;
          $arrFound['valid_name']         [$valueFound->synonym_id] = $valueFound->valid_name;
          break;
        case "names_valid":# @annotation: $_POST[doSearch] names_valid
          $arrFound['text']            [$valueFound->valid_id] = $valueFound->notes;
          $arrFound['valid_id']        [$valueFound->valid_id] = $valueFound->valid_id;
          $arrFound['taxon']           [$valueFound->valid_id] = $valueFound->taxon;
          $arrFound['subfamily']       [$valueFound->valid_id] = $valueFound->subfamily;
          $arrFound['valid_author_id'] [$valueFound->valid_id] = $valueFound->valid_author_id;
          $arrFound['valid_author']    [$valueFound->valid_id] = $valueFound->valid_author;
          break;
        case "references":# @annotation: $_POST[doSearch] names_valid
          $arrFound['text']     [$valueFound->id] = $valueFound->author_names;
          $arrFound['text']     [$valueFound->id].= " ".$valueFound->year.$valueFound->literal.": ";
          $arrFound['text']     [$valueFound->id].= " «".$valueFound->title."» ";
          $arrFound['text']     [$valueFound->id].= " ".$valueFound->journal." ";
          $arrFound['text']     [$valueFound->id].= (empty($valueFound->editor_name)? "" :  " **EDITOR**: ".$valueFound->editor_name." ");
          $arrFound['text']     [$valueFound->id].= (empty($valueFound->abstract)? "":" **ABSTRACT**: ".$valueFound->abstract." ");
          $arrFound['text']     [$valueFound->id].= (empty($valueFound->keywords)? "":" **KEYWORDS**: ".$valueFound->keywords." ");
          $arrFound['text']     [$valueFound->id].= (empty($valueFound->notes)? "":" **NOTES**: ".$valueFound->notes." ");
          $arrFound['reference'][$valueFound->id] = "<strong>".$valueFound->author_names."";
          $arrFound['reference'][$valueFound->id].= " (".$valueFound->year.$valueFound->literal.")</strong>: ";
          $arrFound['reference'][$valueFound->id].= (empty($valueFound->title)?       "" : " <em>".$valueFound->title."</em>.");
          $arrFound['reference'][$valueFound->id].= (empty($valueFound->journal)?     "" :  " ".$valueFound->journal.".");
          $arrFound['reference'][$valueFound->id].= (empty($valueFound->volume)?      "" :  " Vol: ".$valueFound->volume);
          $arrFound['reference'][$valueFound->id].= (empty($valueFound->number)?      "" :  " No: ".$valueFound->number);
          $arrFound['reference'][$valueFound->id].= (empty($valueFound->page)?        "" :  ": ".$valueFound->page.". ");
          $arrFound['reference'][$valueFound->id].= (empty($valueFound->editor_name)? "" :  " Editor: ".$valueFound->editor_name.". ");
          $arrFound['id']       [$valueFound->id] = $valueFound->id;
          break;
      }// end switch
    }// end foreach objResult
    // count results
    $intN=count($arrFound['text']);
  # @annotation $_POST[doSearch]: start content (aditional info)
    $content.="<div class='info'> <p style='text-align:justify;'>".$intN." result".($intN>1?"s":"").".</p> ";
    switch ($_POST['table']) {
      case "characters":# @annotation $_POST[doSearch]: content (info characters)
        $content.="<p style='text-align:justify;'>Note that some links point to the MySQL database directly and may clicked <span class='warning'>twice</span> to get the actual data set properly altered. See also the Openoffice database interface in folder http://localhost/".$config['ServerRootDir']."key/characters_chiros.odb where you can create your own graphical user interface (GUI) to query for numerical characters.</p>";
        break;
      case "images":# @annotation $_POST[doSearch]: content (info images)
        $content.="<p style='text-align:justify;'>Note to delete images use menu '<img src='../structure/img/b_systree.png' align='middle' > Manage Names'.</p>";
        break;
      #case "synonyms":
      #break;
    }// end switch
    $content.="</div>\n";

  # @annotation $_POST[doSearch]: compose list with content
    $content.="<ol class='searchResults'>";
    foreach($arrFound['text'] as $keyFound  => $valueFound){
      switch ($_POST['table']) {
        case "characters":# @annotation $_POST[doSearch]: compose list with content (characters)
        # uses id_characters as keyFound
          if(preg_match('#sp$#',$arrFound['taxon'][$keyFound])){# go to genus page
            $strHtml = "href='chirokey.php?genus_valid_id=".$arrFound['genus_valid_id'][$keyFound]."&amp;show=genus&morph_attrib_id=%60names_subfamily%60.%60morph_attrib_id%60' target='chirokey_larvae' title='Go to page: ".$arrFound['taxon'][$keyFound]."'";
            $strTaxon = $objHtml->emphase($arrFound['taxon'][$keyFound], $_POST['words'], "highlightSearch");
            $strText  = ": ".$objHtml->emphase($arrFound['text'][$keyFound], $_POST['words'], "highlightSearch");
          }else {# go to either phpmyadmin2-11-4 or to specimen page
            $strHtml ="href = 'http://localhost/".$config['phpmyadminpath']."/tbl_change.php?db=chirokey_larvae&amp;table=characters&amp;token=".$_SESSION['PMA_single_signon_token']."&amp;primary_key=+%60characters%60.%60id%60+%3D+".$arrFound['id_characters'][$keyFound]."&amp;sql_query=SELECT+%2A++FROM+%60characters%60+WHERE+%60names_valid_id%60+%3D+".$arrFound['genus_valid_id'][$keyFound]."&amp;goto=sql.php' target='_blank' class='external' title='modify numeric data in a new window for in MySQL database'";
            $strTaxon = "<img src='../structure/img/tag_table_edit.png' width='22' height='22' align='middle'>";
            $strText   = " <em>".$objHtml->emphase($arrFound['taxon'][$keyFound], $_POST['words'], "highlightSearch")."</em> ".$arrFound['descriptor'][$keyFound]." from ".$arrFound['author'][$keyFound];
              $strText.= " <span title='modify web page data'> <a class='nothing' target='chirokey_larvae'  href='chirokey.php?id_characters=".$arrFound['id_characters'][$keyFound]."&amp;morph_attrib_id=%60names_subfamily%60.%60morph_attrib_id%60&amp;show=specimen&amp;valid_id=".$arrFound['genus_valid_id'][$keyFound]."'><img src='../structure/img/b_edit_page.png' align='middle' width='22' height='22' >    </a>\n  </span>:<br>";
              $strText.=$objHtml->emphase($arrFound['text'][$keyFound], $_POST['words'], "highlightSearch");
          }
        break;
        case "images":# @annotation $_POST[doSearch]: compose list with content (images)
          $strHtml = "";
          // $strText = "\n<form action='' method='POST' style='padding=0px;margin=0px;'>\n";
          $strText ="<input type='hidden' value='1' name='EditDeleteImages'>";
          $strText.="
          <span class=\"overlib\" onmouseover=\"return overlib('<img align=\'middle\'  src=\'../../".$config['dirSpec'].$arrFound['filename'][$keyFound]."\' >', FULLHTML, ABOVE, RIGHT,CENTER );\" onmouseout=\"return nd();\">
          <em>"
          .$objHtml->emphase($arrFound['taxon'][$keyFound], $_POST['words'], "highlightSearch").
          "</em>
          </span>
          ".$arrFound['descriptor'][$keyFound]."
          <a href='synonyms.php' target='chirokey_larvae' title='manage specimen&#039;s name' class='nothing'><img src='../structure/img/b_systree.png' align='middle' ></a>
          (".$arrFound['morph_long'][$keyFound].")
          <span class='nothing' title='modify specimen&#039;s image' onClick=\"window.open('image.php?action=edit&popup=1&file=".$arrFound['filename'][$keyFound]."&specimen=".$arrFound['taxon'][$keyFound]."&descriptor=".$arrFound['descriptor'][$keyFound]."&morph_long=".$arrFound['morph_long'][$keyFound]."&image_id=".$arrFound['image_id'][$keyFound]."', '','toolbar=no,status=yes,menubar=no,scrollbars=yes,width=900,height=550')\" >
          <img id='pencilEdit' src='../structure/img/b_edit.png' width='16' height='16' >
          </span>"
          .$objHtml->emphase($arrFound['text'][$keyFound], $_POST['words'], "highlightSearch").
          "
          <br>(added:&nbsp;".$arrFound['time_added'][$keyFound]." ".$arrFound['time_modified'][$keyFound].")
          <br>
        <span class='slightlyNote'>From "
          .$objHtml->emphase($arrFound['authors_short'][$keyFound], $_POST['words'], "highlightSearch").
          ": <em>"
          .$objHtml->emphase($arrFound['title'][$keyFound], $_POST['words'], "highlightSearch").
          "</em>, ".$arrFound['journal'][$keyFound].".
        </span>
        ";
        break;
      case "synonyms":# @annotation $_POST[doSearch]: compose list with content (synonyms)
          $synonym_author = $arrFound['synonym_author_id'][$keyFound] == 0 ? "<span class=\"attention\">".$arrFound['synonym_author'][$keyFound]."</span>" :  "<a href=\"http://".$_SERVER['SERVER_NAME']."/".$config['ServerRootDir'].$config['ServerKeyDir']."chirokey.php?show=references#refid".$arrFound['synonym_author_id'][$keyFound]."\" class=\"reference\" target=\"chirokey\">".$arrFound['synonym_author'][$keyFound]."</a>";
          $valid_author   = $arrFound['valid_author_id'][$keyFound]   == 0 ? "<span class=\"attention\">".$arrFound['valid_author'][$keyFound]."</span>" :  "<a href=\"http://".$_SERVER['SERVER_NAME']."/".$config['ServerRootDir'].$config['ServerKeyDir']."chirokey.php?show=references#refid".$arrFound['valid_author_id'][$keyFound]."\" class=\"reference\" target=\"chirokey\">".$arrFound['valid_author'][$keyFound]."</a>";
          $synonym_name = "<span class=\"search\" title=\"Click to search &#187;".$arrFound['synonym_name'][$keyFound]."&#171;\" onclick=\"check=confirm('You clicked on &#187;".$arrFound['synonym_name'][$keyFound]."&#171;.\\nDo you wish to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=".str_replace(" ","+",$arrFound['synonym_name'][$keyFound])."&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species' >".$arrFound['synonym_name'][$keyFound]."</em></span>";
          $valid_name   = "<span class=\"search\" title=\"Click to search &#187;".$arrFound['valid_name'][$keyFound]."&#171;\" onclick=\"check=confirm('You clicked on &#187;".$arrFound['valid_name'][$keyFound]."&#171;.\\nDo you wish to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=".str_replace(" ","+",$arrFound['valid_name'][$keyFound])."&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species' >".$arrFound['valid_name'][$keyFound]."</em></span>";
          //edit
          $edit="<a class='nothing' title='modify entry' target='_blank' href='http://localhost/".$config['phpmyadminpath']."/tbl_change.php?db=chirokey_larvae&table=names_synonyms&token=".$_SESSION['PMA_single_signon_token']."&primary_key=+%60names_synonyms%60.%60id%60+%3D+".$arrFound['synonym_id'][$keyFound]."&sql_query=SELECT+%2A+FROM+%60names_synonyms%60&goto=sql.php'><img src='../structure/img/b_edit.png' width='16' height='16' ></a>";
        $strText = $edit." ".$synonym_name." ".$synonym_author."  valid as ";
        $strText.= $valid_name." ".$valid_author ;
          $note = empty($arrFound['text'][$keyFound]) ? '' : "<br>Note: ";
        $strText.=$note.$objHtml->emphase($arrFound['text'][$keyFound], $_POST['words'], "highlightSearch");
        break;
      case "names_valid":# @annotation $_POST[doSearch]: compose list with content (names_valid)
        $valid_author   = $arrFound['valid_author_id'][$keyFound]   == 0 ? " <span class=\"attention\">".$arrFound['valid_author'][$keyFound]."</span>" :  " <a href=\"http://".$_SERVER['SERVER_NAME']."/".$config['ServerRootDir'].$config['ServerKeyDir']."chirokey.php?show=references#refid".$arrFound['valid_author_id'][$keyFound]."\" class=\"reference\" target=\"chirokey\">".$arrFound['valid_author'][$keyFound]."</a>";
        $taxon   = "<span class=\"search\" title=\"Click to search &#187;".$arrFound['taxon'][$keyFound]."&#171;\" onclick=\"check=confirm('You clicked on &#187;".$arrFound['taxon'][$keyFound]."&#171;.\\nDo you wish to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=".str_replace(" ","+",$arrFound['taxon'][$keyFound])."&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species' >".$objHtml->emphase($arrFound['taxon'][$keyFound], $_POST['words'], "highlightSearch")."</em></span> $valid_author, ".$arrFound['subfamily'][$keyFound];
      //edit
        $edit="<a class='nothing' title='modify entry' target='_blank' href='http://localhost/".$config['phpmyadminpath']."/tbl_change.php?db=chirokey_larvae&table=names_valid&token=".$_SESSION['PMA_single_signon_token']."&primary_key=+%60names_valid%60.%60id%60+%3D+".$arrFound['valid_id'][$keyFound]."&sql_query=SELECT+%2A+FROM+%60names_valid%60&goto=sql.php'><img src='../structure/img/b_edit.png' width='16' height='16' ></a> ";
        $note = empty($arrFound['text'][$keyFound]) ? '' : $objHtml->emphase($arrFound['text'][$keyFound], $_POST['words'], "highlightSearch");
        $strText = $edit.$taxon.$note;
        break;
      case "references":# @annotation $_POST[doSearch]: compose list with content (names_valid)
        $strHtml = " target='chirokey_larvae' class='nothing' title='edit this record' href='update.php?title=references&amp;refid=".$arrFound['id'][$keyFound]."' ";
        $strTaxon = "<img align='middle' id='pencilEdit' src='../structure/img/b_edit.png' width='16' height='16' > ";
        $strPreText = $arrFound['reference'][$keyFound]."<br>";
        $strText    = $objHtml->emphase($arrFound['text'][$keyFound], $_POST['words'], "highlightSearch");
        break;
      }// end switch

    # @annotation $_POST[doSearch]: (final) list
      $content.="\n  <li><a ".$strHtml.">".$strTaxon."</a>".$strPreText.$strText.$strPostText."</li>";
    }// end foreach
    $content."</ol>";
  # @annotation print content
    echo $content;
  }// end something was found
?>

</div><!-- end class content -->
</body>
</html>