<?php
/**
 * Tools available for the whole program
 *
 * Some tools that are needed elsewhere in the key
 *
 * @author Andreas Plank <andreas.plank@web.de>
 * @version 2009-02-24 21:24:33
 * @license Noncommercial-Share Alike 3.0 Unported
 * @todo minor issue: document further detailed
 * @todo highlight references for species e.g. Ablabesmyia (Ablabesmyia) aspera
 * @package chirokey_larvae
 * @subpackage program
 */


/**
 * Prevent dublicate entries in database by a reload
 *
 * Don't forget to initialize the session!
 *
 * @package chirokey_larvae
 * @subpackage program
 * @todo Seems to work. Example see execute.php
 * @category program tools
 */

class formreload {/* prevent dublicate entries in database */
  /**
  * saveing of tokens
  * @var    array
  * @access    private
  */
  private $tokenarray = '__token';
  /**
  * name of hidden element <input>
  * @var        string
  * @access    public
  */
  public $tokenname = '__token';

  /**
  * generates a random uniq id
  *
  * Generation of tokens like 204611b9eb74d7d3e3ecbe146313c7a9 each time the form is loaded.
  * As <var>$_SESSION</var> saves old tokens, a comparison reveal if there was a reload or not.
  * Evaluating with {@link formreload::easycheck_empty()}
  *
  * @return HTML hidden <input> for any form
  */

  public function get_formtoken() {
    $tok = md5(uniqid("foobarmagic"));
    $content.=  sprintf("\n<input type='hidden' name='%s' value='%s'>", $this->tokenname, htmlspecialchars($tok));
    return $content;
    //writes at %s in this order $this->tokenname and htmlspecialchars($tok)
  }
  /**
  * check saved tokens with actual ones
  *
  * @return boolean : false=reload true=no reload
  */
  public function easycheck_empty(){
    $info.= "easycheck_empty() SESSION before ".print_pre($_SESSION, true);
    $tok = $_POST[$this->tokenname];
    if (isset($_SESSION[$this->tokenarray][$tok])) {
      $info.= "easycheck_empty() SESSION after ".print_pre($_SESSION, true);
      $info .= "reload";
      // print_pre($info);
      return false;
      # reload
    } else {
      $_SESSION[$this->tokenarray][$tok] = true;
      $info.= "easycheck_empty() SESSION after ".print_pre($_SESSION, true);
      $info .= "no reload";
      //   print_pre($info);
      return true;
      # no reload
    }
  }
}// end class formreload



/**
 * for dealing with HTML (formatting)
 *
 * @package chirokey_larvae
 * @subpackage program
 * @category html
 */
class html
{
  /*
  public - überall zugänglich, lesen, verändern
  protected - innerhalb und in abgeleiteter Klasse
  private - NUR innerhalb Klasse
  */
  public $array;
  public $name; // html-Name
  private $default; // z.B: selected='true'
  private $structure; // z.B.: onselect, etc
  private $preselected; // value or option
  public $doOptionGroup = FALSE; // value or option
/**
 * creates select options
 *
 * @param array $arrValueToOption an array with [value] -> [option]
 * @param string $name element's name
 * @param string $default a default (preselected) option or a value. Produces selected='true'
 * @param string $structure some additional settings for HTML-selection, e.g. JavaScript
 * @param string $preselected some additional settings for HTML-selection, e.g. JavaScript
 * @return HTML <samp><select>...</select></samp>
 */
  public function select($arrValueToOption, $name, $default='',$structure='', $preselected='option'){
  $content="\n<select name=\"".$name."\" ".$structure.">\n";
  if($this->doOptionGroup){
    $n = count($arrValueToOption);
    $i=1;
  }
  foreach ($arrValueToOption as $value => $option) {
    // $var = condition ? TRUE : FALSE or condition ? TRUE : FALSE
    $value=str_replace("'", "'", $value);
    if($this->doOptionGroup){
      $letter = substr($option,0,1);
      if($letter!=$letterlast){
        $content.=($i>1)?  "\n  </optgroup>" :"";
        $content.="\n  <optgroup label=\"".$letter."\">";
      }
    }
    switch ($preselected) {
      case 'value':
        $content.= $default===$value ? "\n    <!-- selected--><option value=\"".$value."\" selected=\"true\" >".$option."</option>" : "\n    <option value=\"".$value."\">".$option."</option>";
        break;
      default:
        $content.= $default==$option ? "\n    <!-- selected--><option value=\"".$value."\" selected=\"true\" >".$option."</option>" : "\n    <option value=\"".$value."\">".$option."</option>";
    }// end switch
    if($this->doOptionGroup){
      $letterlast = substr($option,0,1);
      $content.= ($i==$n)? "\n  </optgroup>\n" :"";
    }
    $i++;
  }// end foreach
  return $content.="\n</select>";
  }// end function select

  public $text;
  public $searchtext;
  public $contenttype;
  public $isHref;
   // $href = 'http://';//.$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI'].'#refid';

/**
 * Emphase genera, references ...
 *
 * Highlighting for several cases: genera, references, maps, put references into $_SESSION variable with {@link putToArray()}
 * @param string $originaltext original text, e.g. in wikipedia like syntax
 * @param mixed  $replacetext text to replace
 * @param string $contenttype for switch different cases, e.g. idtogenus
 * @param boolean $isHref true: use REQUEST_URI (make links to page itself); false: 'chirokey.php?show=references' (whole reference list)
 * @return HTML
 */

  function emphase(&$originaltext='',$replacetext='',$contenttype='',$isHref = true)  {
    /**
    * Extract current server path relatively
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
    switch ($contenttype) {
      /*
        "idtogenus"  highlight genus in text
        "idtogenustext"
        "references"
        "species"
        "genus"
        "maps"
        "languages"
        "strImgToImage" Img:File.jpg -> <img ...>
      */
    case "idtogenus":
    # @annotation highlight genus in text
      $originaltext=split(" ",$originaltext);
      foreach($originaltext as $value){
        $text.=!empty($value)? "<a target='chirokey_larvae' href='chirokey.php?show=genus&amp;morph_attrib_id=".$_GET['morph_attrib_id']."&amp;genus_valid_id=".$value."'><em>".$replacetext[$value]."</em></a> ":"";
      }
        return $text;
      break;
    case "idtogenustext":# @annotation: emphase(idtogenustext)
      $originaltext=split(" ",$originaltext);
      foreach($originaltext as $value){
        #echo $value."|";
        $text.= !empty($value)? $replacetext[$value]." ":"";
      }
      return $text;
      break;
    case "references":  # @annotation: emphase(references)
      foreach($replacetext as $key => $value){
        $valuehtml=rawurlencode($value);
        $link = $isHref ? "href='http://".$_SERVER['SERVER_NAME'].$_SERVER['REQUEST_URI']."#refid".$key."'": "href='http://localhost/".$config['ServerRootDir']."key/chirokey.php?show=references#refid".$key."'";
        $value_searched=array(
          "/\{citeID\|".$key."\|(.+)\}/U",
          "/\{textciteID\|".$key."\|(.+)(\d+[a-z]*)\}/U",
          "/\{textciteID\|".$key."\|(.+)(n\.\s*d\.\s*)\}/U", // n.d.
          "/\{citeyearID\|".$key."\|(.+)\}/U",
          "/\{citeauthorID\|".$key."\|(.+)\}/U",
          "/\{nociteID\|".$key."\|.+\}/Ue",
          "/\{numciteID\|".$key."\|(.+)\}/U"
        );
        $value_replaced=array(
          "<a title='Click for reference: \\1.' class='reference' target='chirokey_larvae' ".$link." >".$value."</a>",
          "<a title='Click for reference: \\1\\2.' class='reference' target='chirokey_larvae' ".$link." >".preg_replace("#(.+?)(\d+[a-z]*$)#","\\1 (\\2)",$value)."</a>",
          "<a title='Click for reference: \\1\\2.' class='reference' target='chirokey_larvae' ".$link." >".preg_replace("#(.+?)(n\.\s*d\.\s*$)#","\\1 (\\2)",$value)."</a>",
          "<a title='Cited year of reference: \\1.' class='reference' target='chirokey_larvae' ".$link." >".preg_replace("/.+([0-9]{4}[a-z]?)/","\\1",$value)."</a>",
          "<a title='Cited author of reference: \\1.' class='reference' target='chirokey_larvae' ".$link." >".preg_replace("/(.+)\s*[0-9]{4}[a-z]?/","\\1",$value)."</a>",
          "''.putToArray(".$key.",\$_SESSION[\"references\"]).''",
          "<a title='Click for reference: \\1.' class='reference' target='chirokey_larvae' ".$link." >".$key."</a>"
          );
        $originaltextnew=preg_replace($value_searched, $value_replaced, $originaltext);
        if($originaltext!=$originaltextnew)  $_SESSION['references'][] = $key;
        $originaltext=$originaltextnew;
      }
      return $originaltext;
      break; // end references
    case "species" : # @annotation: emphase(species)
      foreach($replacetext as $value){
        if(preg_match("/".$value."/",$originaltext)  && !empty($value)){
          $value=str_replace("?","\?",$value);
          $search=array(//pay attention to '' values in database
          /*1*/  "/([A-Z]{1}\.?\s*\([A-Z]{1}\.?\))(\s*)\b(".($value==="sp"?"sp{1,2}":$value).")\b/",# A (A) species
          /*2*/  "/([A-Z]{1}\.)\s*(cf\.|gr\.|nr\.|agg\.)*\s*\b(".($value==="sp"?"sp{1,2}":$value).")\b/",# A cf|gr species
          /*3*/  "/\<em\>([a-z]+)(\<\/em\>\<\/a\>)\s*(cf\.|gr\.|nr\.|agg\.)*\s*\b(".($value==="sp"?"sp{1,2}":$value).")\b/i", # emphased genus cf.|gr. species
          /*4*/  "/\<em\>([a-z]+)(\<\/em\>\<\/a\>)\s*(\([a-z]+\.?\))\s*(cf\.|gr\.|nr\.|agg\.)*\s*\b(".($value==="sp"?"sp{1,2}":$value).")\b/i", # emphased genus (subgenus) cf.|gr. species
            );
          $replace=array(/* http://www.google.com/search?rls=en&q=\\1+\\3&ie=utf-8&oe=utf-8
          check=confirm('Delete this image from database and species directory?\\n%2\$s');if (check==false) return false */
            "<span class=\"search\" title=\"Click to search &#187;\\1 \\3&#171;\" onclick=\"check=confirm('You clicked on &#187;\\1 \\3&#171;.\\nDo you like to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=\\1+\\3&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species' >\\1&nbsp;\\3</em></span>",
            "<span class=\"search\" title=\"Click to search &#187;\\1 \\2\\3&#171;\" onclick=\"check=confirm('You clicked on &#187;\\1 \\2\\3&#171;.\\nDo you like to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=\\1+\\2\\3&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species'>\\1&nbsp;\\2\\3</em></span>",
            "<em>\\1\\2&nbsp;<span class=\"search\" title=\"Click to search &#187;\\1 \\4&#171;\" onclick=\"check=confirm('You clicked on &#187;\\1 \\4&#171;.\\nDo you like to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=\\1+\\4&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species' >\\3\\4</em></span>",
            "<em>\\1\\2&nbsp;<span class=\"search\" title=\"Click to search &#187;\\1 \\5&#171;\" onclick=\"check=confirm('You clicked on &#187;\\1 \\5&#171;.\\nDo you like to search this string in a new window?');if (check==false) return false; if (check==true) window.open('http://www.google.com/search?rls=en&amp;q=\\1+\\5&amp;ie=utf-8&amp;oe=utf-8');\"><em class='species' >\\3 \\4\\5</em></span>",
            );
          $originaltext=preg_replace( $search, $replace,$originaltext);//Ersetzung: M.fusca, M. fusca, fusca
        }// end if
      }
      return $originaltext;
      break; // end species
    case "genus":  # @annotation: emphase(genus)
      foreach($replacetext as $id => $value){
        $value = htmlentities($value,ENT_QUOTES,'UTF-8');
        if(preg_match("/".$value."/",$originaltext)  && !empty($value)){
          $search=array(//pay attention to '' values in database
          // ?<!alt= means look before NOT (?<!) 'alt='
            "/(?<!alt=')(?<!alt=\\\')(?<!title=')(?<!title=\\\')(?<!\()(?<!=)(?<!\+)(?<!--)(?<!-- )(?<!--  )($value)/",
            /* find with NOT «title='» and so on before genusname, so the finding reference
              has to be \\1 and not \\2. ONLY FIXED PATTERNS POSSIBLE! not « +» */
            );
          $replace=array(
          "<a target='chirokey_larvae' href='chirokey.php?show=genus&amp;morph_attrib_id=".$_GET['morph_attrib_id']."&amp;genus_valid_id=".$id."'><em>\\1</em></a>",
             // "<em class='species' >\\1</em>",
            );
          $originaltext=preg_replace( $search, $replace,$originaltext);//Ersetzung: M.fusca, M. fusca, fusca
        }// end if
      }// end foreach
      return $originaltext;
      break; // end genus
    case "highlightSearch":
      $search = array(
        #'#&lt;(.+)&gt;#U', # 1 HTML
        '#\{(citeID|textciteID|nociteID|numciteID|citeyearID)\|[0-9]+\|(.+)\}#U',    # 2 citations
        '#\{overlib\|([^|]+)\|[^|]*\}#U', # 3 overlib
        "#^\{\|.*$#m",                    # 4 table body
        "#^\|\}.*$#m",                    # 5 table body
        "#^\|\-.*#m",                     # 6 table row
        '#^\|(.+)$#m',                    # 7 table data
        '#^\!im[0-9]*\|(.+)$#m',          # 8 table head
        '#^\!\|(.+)$#m',                  # 9 table head
        "#(".$replacetext.")#ui", # 10 highlght
      );
      $replace = array(
        #'<\\1>',    # 1 HTML
        "\\2", # 2 citations
        "\\1", # 3 overlib
        "",    # 4 table body
        "",    # 5 table body
        "",    # 6 table row
        "\\1", # 7 table data
        "\\1", # 8 table head
        "\\1", # 9 table head
        "<span class='highlight'>\\1</span>", # 10 highlight
      );
    $textcopy = preg_replace( $search, $replace,$originaltext);
      return $textcopy;
      break;
    case "maps":  # @annotation: emphase(maps)
       // foreach($replacetext as $id => $value){
         // if(preg_match("/".$value."/",$originaltext)  && !empty($value)){
         $settingsSearch = array(
          '#&amp;#',
          '#^(.+)$#'
          );
         $settingsReplace = array(
         '; var ',
         ' var \\1;'
         );
          $search=array(
/* 1 */ "/\{\{openLayer\|([^|]+)\|(.*)[\\r\\n](.+)\}\}/Use", //
            );
          $replace=array(
/* 1 */  "'<script src=\"http://maps.google.com/maps?file=api&amp;v=2&amp;key=ABQIAAAA9XNhd8q0UdwNC7YSO4YZghSPUCi5aRYVveCcVYxzezM4iaj_gxQ9t-UajFL70jfcpquH5l1IJ-Zyyw\"></script>
<div class=\"openlayermap\" id=\"map\\1\" title=\"openLayer map no: \\1\"></div>
<script type=\"text/javascript\" src=\"http://www.openlayers.org/dev/lib/OpenLayers.js\"></script>
  <script type=\"text/javascript\">
  //zoom=7&lat=33.48633&lon=94.48242&layers=TB000
  // '.preg_replace(\$settingsSearch, \$settingsReplace, '\\2').'
    map\\1 = new OpenLayers.Map(\"map\\1\", { controls: [] });

    map\\1.addControl(new OpenLayers.Control.PanZoom()); // zoom default
  //  map\\1.addControl(new OpenLayers.Control.PanZoomBar()); //Zoom Scale
    map\\1.addControl(new OpenLayers.Control.MouseToolbar());
    map\\1.addControl(new OpenLayers.Control.LayerSwitcher());
    map\\1.addControl(new OpenLayers.Control.Permalink());
    map\\1.addControl(new OpenLayers.Control.ScaleLine());
    map\\1.addControl(new OpenLayers.Control.MousePosition({prefix:\"Lon:&nbsp;\",separator:\"&nbsp;|&nbsp;Lat:&nbsp;\",suffix:\"\"}));
    //  map\\1.addControl(new OpenLayers.Control.OverviewMap()); // does not work
    map\\1.addControl(new OpenLayers.Control.KeyboardDefaults());


    var ol_wms = new OpenLayers.Layer.WMS( \"World Map (metacarta)\",
      \"http://labs.metacarta.com/wms-c/Basic.py?\", {layers: \"basic\", format: \"image/png\" } );

    var jpl_wms = new OpenLayers.Layer.WMS( \"Satellite (metacarta)\",
      \"http://labs.metacarta.com/wms-c/Basic.py?\", {layers: \"satellite\", format: \"image/png\" } );

    var nasa = new OpenLayers.Layer.WMS( \"NASA Global Mosaic\",
            \"http://t1.hypercube.telascience.org/cgi-bin/landsat7\",
            {layers: \"landsat7\"});
    var google = new OpenLayers.Layer.Google( \"Google Hybrid\" , {type: G_HYBRID_MAP });
    // points layer
    var newl = new OpenLayers.Layer.Text( \"Points\", {location: \"openlayerMaps.php?map=map\\1\"} );
    map\\1.addLayer(newl);
    map\\1.addLayers([ol_wms, jpl_wms, nasa,google]);

//     if(zoom & lon & lat)
//       map\\1.setCenter(new OpenLayers.LonLat(lon, lat), zoom);
    if (!map\\1.getCenter())
      map\\1.zoomToMaxExtent();
    '.putToArray('\\3',\$_SESSION[\"map\\1\"]).'
    // setVisibility(\'map\\1\',false);
</script>'\n",// non breaking space
            );
      $originaltext=preg_replace( $search, $replace,$originaltext);//Ersetzung: M.fusca, M. fusca, fusca
      return $originaltext;
      break; // end maps
    case "strImgToImage":
      foreach ($replacetext as $key => $value) {
        $originaltext = str_replace($key, $value, $originaltext);
      }// end foreach
      return $originaltext;
      break;
    case "strTemplateToHtml":
      foreach ($replacetext as $key => $value) {
        $originaltext = str_replace("Template:".$key, $value, $originaltext);
      }// end foreach
      return $originaltext;
      break;
    }// end switch
  }// end function emphase


/**
 * create HTML from wikipedia like syntax
 *
 * Highlighting the whole page stored in database table 'characters':
 * lists, tables, sections, overlib features (JavaScript library), special characters, ....
 * @param string $text original text, e.g. in wikipedia like syntax
 * @return HTML
 */

  function pattern2tag($text='')//@annotation: pattern2tag(text) special patterns:
    {//echo "<em>pattern2tag:</em>";print_r($this->text);
    /**
    * Extract current server path relatively
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
/* */
    $searchpattern=array(
/*  1 */  "/([\r\n]){3,}/",                               /* 1 line break -> |*/
/*  2 */  "/^(\{\|)(\&lt;!\-\-.+\-\-\&gt;)*(.*)[\n\r]$/m",/* 2 table start */
// (foo)((?:(?!foo).)+)(bar)
// (^([^\|]+))((?:(?!^([^\|]+)).)+)(^\|\}$)
/*  3 */  "/^\|\}[\n\r]$/Um",                          /* 3 table end */
/*  4a */ "/(^\!im)(\d{1,2})(.*?)(\|)(.+)[\n\r]$/me",      /* 4a th */
/*  4b */ "/(^\!im)(.*?)(\|)(.+)[\n\r]$/me",               /* 4b th */
/*  5 */  "/(^\!)([^|]*)(\|*)(.*)/m",                       /* 5 th */
/*  6 */  "/^(\\|\\-)(.*)[\n\r]$/m",                       /* 6 tr (^[^|]+)*/
/*  7a */  "/(^<table.*>)(.+)\s*<\/tr>/Ums",          /* 7 tr correct */
/*  7b */  "/(<\/td>)(.+)(<\/tr>)(.+)(<\/table>)/Um", /* 7 tr correct e.g. <tbody>*/
// (foo)((?:(?!foo).)+)(bar)
// (</t[dh]>)((?:(?!</t[dh]>).)+)(</tr>)
/*  7c */  "#(</t[dh]>)((?:(?!</t[dh]>).)+)([\n\r]\s+</tr>)#Usm", /* 7 tr correct e.g. <tbody>*/
// /*  7c */  "/([\n\r])+(<\/tr>)(\W+)(<tr[^>]+>)/Usm", /* 7 tr correct e.g. <tbody>*/
/*  8a*/  "/(^\|)(.*(?<![\n\r]))/m",                   /* 8 td */
/*  9 */  "/(\&lt;!\-\-)(.+)(\-\-\&gt;)/",  /* 9 comment */
/* 10 */  "/(''''')(.+?)(''''')/",          /* 10emphased & strong*/
/* 11 */  "/(''')(.+?)(''')/",              /*11 emphased */
/* 12 */  "/('')(.+?)('')/",                /*12 strong */
/* 13 */  "/(\[{2})(\S+?)(\]{2})/",         /*13 link without space */
/* 14 */  "/(\[{2})(\S+?)(\s)(.+?)(\]{2})/",/*14 link with space */
/* 15 */  "/(^\*{4})\s*(.*?$)/m",           /*15 unordered list level 4 */
/* 16 */  "/(^\*{3})\s*(.*?$)/m",           /*16 unordered list level 3 */
/* 17 */  "/(^\*{2})\s*(.*?$)/m",           /*17 unordered list level 2 */
/* 18 */  "/(^\*)\s*(.*?$)/m",              /*18 unordered list level 1 */
/* 19 */  "/^<\/ul>\n<ul (class='(point|circle|minus|disc)')>\n/m", /*19 unordered list remove <\/ul>\n<ul>*/
/* 20 */  "/^\s{2}<\/ul>\n\s{2}<ul\s*(class='(point|circle|minus|disc)')>\n/m", /*20 unordered list remove <\/ul>\n<ul>*/
/* 21 */  "/^\s{4}<\/ul>\n\s{4}<ul\s*(class='(point|circle|minus|disc)')>\n/m", /*21 unordered list remove <\/ul>\n<ul>*/
/* 22 */  "/^\s{6}<\/ul>\n\s{6}<ul\s*(class='(point|circle|minus|disc)')>\n/m", /*22 unordered list remove <\/ul>\n<ul>*/
/* 22a */ "/^\#~/m",              /*26 ordered list level 1: white space  */
/* 23 */  "/(^\#{4})\s*(.*?$)/m",           /*23 ordered list level 4 */
/* 24 */  "/(^\#{3})\s*(.*?$)/m",           /*24 ordered list level 3 */
/* 25 */  "/(^\#{2})\s*(.*?$)/m",           /*25 ordered list level 2 */
/* 26 */  "/(^\#)\s*(.*?$)/m",              /*26 ordered list level 1 */
/* 27 */  "/^<\/ol>\n<ol (class='(decimal|alpha|romanupper|alphaupper)')>\n/m", /*27 ordered list remove <\/ol>\n<ol>*/
/* 28 */  "/^\s{2}<\/ol>\n\s{2}<ol\s*(class='(decimal|alpha|romanupper|alphaupper)')>\n/m", /*28 ordered list remove <\/ol>\n<ol>*/
/* 29 */  "/^\s{4}<\/ol>\n\s{4}<ol\s*(class='(decimal|alpha|romanupper|alphaupper)')>\n/m", /*29 ordered list remove <\/ol>\n<ol>*/
/* 30 */  "/^\s{6}<\/ol>\n\s{6}<ol\s*(class='(decimal|alpha|romanupper|alphaupper)')>\n/m", /*30 ordered list remove <\/ol>\n<ol>*/
/* 31 */  "/(^\;)\s*(.*?)\n{0,1}\:\n{0,1}(.*?$)/m",              /* definition */
/* 32 */  "/^<\/dl>\n<dl>/m",              /* multiple definitions */
// /* 33 */  "/(\d+)(st|nd|rd|th)/",      /* superscript */ // destroys text images
/* 36 */  "/(\s|~)-&gt;(\s|~)/", // -> right arrow
/* 37 */  "/(\s|~)&lt;-(\s|~)/", // &lt;- left arrow
/* 38 */  "/(\s|~)&lt;-&gt;(\s|~)/", // left right arrow
/* 39 */  "/(\s|~)=&gt;(\s|~)/", // double right arrow
/* 40 */  "/(\s|~)&lt;=(\s|~)/", // double left arrow
/* 41 */  "/(\s|~)&lt;=&gt;(\s|~)/", // double left right arrow
/* 42 */  "/\s{1,3}\-\-\s/", // &ndash;
/* 43 */  "/\s{1,3}\-\-\-\s/", // &mdash;
/* 44 */  "/\bfemale\b/i", // male
/* 45 */  "/\bmale\b/i", // female
/* last 1 */"/\&lt;/",  /* < sign */
/* last 2 */"/\&gt;/",  /* > sign */
/* 34 */  "/(\s)\<\=\=(\s)/", // less or equal
/* 35 */  "/(\s)\>\=\=(\s)/", // greater or equal
/* 48 */ "/(\s+|\(|,)\+-(\s+|[0-9])/",// plusmn
/* 49 */ "/()\!{2}()/", // <th>
/* 50 */ "/()\|{2}()/", // <td>
/* 51 */ "/^(?>([=]+))(.+[^=])(?>([=]+))$/me", // sections <h1> to <a> e-evaluate as php
/* 52 */ "/^(((<a|<span|\{citeID|\{textciteID|\{numciteID|\{citeyearID)|\w).+)$/mi", // paragraph
/* 53a*/ "/(?<![\/ ])[~]/", // non breaking space (?<!\/)
// /* 53b*/ "/(?=src=')|(?=src=\")(.+)&nbsp;(.+)('|\")/U",
/* 54 */ "/\{(".$config['colornames'].")\}/Ue", // colors
/* 55 */ "/\{overlib\|(.+)\|(.+)\}/Ue", // overlib tool
//         "#(?<!\{\{openLayer)(\<\!\-\-.*)(\{\{openLayer\|(.+)\|(.+)\}\}.*\-\-\>)#Usm"
/* 56 */ "#<!--(?!\{\{openLayer)[^-->]*\{\{openLayer\|([^\|]+)\|.*-->#Usm", // hidden maps
    );

    $replacepattern=array(
/*  1 */   "\n\n",                           /* 1 line break \n -> | */
/*  2 */   "<table \\3>\\2\n",                     /* 2 table start */
/*  3 */   "  </tr>\n\\1\n</table>",                       /* 3 table end */
/*  4a */  "'    <th \\3>\n      <img title=\''.htmlentities('\\5',ENT_QUOTES,'UTF-8').'\' alt=\''.htmlentities('\\5',ENT_QUOTES,'UTF-8').'\'  src=\'image.php?text='.base64_encode('\\5').'&amp;encoding=base64&amp;angle=\\2\'>\n    </th>'",  /* 4a th */
/*  4b */  "'    <th \\2>\n      <img title=\''.htmlentities('\\4',ENT_QUOTES,'UTF-8').'\' alt=\''.htmlentities('\\4',ENT_QUOTES,'UTF-8').'\' src=\'image.php?text='.base64_encode('\\4').'&amp;encoding=base64\'>\n    </th> '",  /* 4b th */
/*  5 */   "    <th \\2>\n      \\4\n    </th>",                /* 4 th */
/*  6 */   "  </tr>\n  <tr \\2>",     /* 6 tr */
/*  7a */   "\\1\\2",                    /* 7 tr correct */
/*  7b */   "\\1\\3\\2\\4\\5",          /* 7 tr correct */
/*  7c */   "\\1\\3\\2",                /* 7 tr correct */
// /*  7c */   "\\1\\2\\3",                /* 7 tr correct */
/*  8a*/   "    <td class='auto' >\\2</td>",    /* 8 td */
/*  9 */   "<!--\\2-->",                     /* 9 comment */
/* 10 */   "<em><strong>\\2</strong></em>",  /* 10emphase strong */
/* 11 */   "<em>\\2</em>",                   /*11 emphase */
/* 12 */   "<strong>\\2</strong>",           /*12 strong */
/* 13 */   "<a href='\\2' class='external' target='_blank'>\\2</a> ", /*13 anchor */
/* 14 */   "<a href='\\2' class='external' target='_blank'>\\4</a> ", /*14 anchor */
/* 15 */   "<ul class='disc'>\n  <ul class='circle'>\n    <ul class='minus'>\n      <ul class='point'>\n        <li>\\2</li>\n      </ul>\n    </ul>\n  </ul>\n</ul>\n",   /*15 ul */
/* 16 */   "<ul class='disc'>\n  <ul class='circle'>\n    <ul class='minus'>\n      <li>\\2</li>\n      </ul>\n  </ul>\n</ul>\n",   /*16 ul */
/* 17 */   "<ul class='disc'>\n  <ul class='circle'>\n    <li>\n      \\2\n    </li>\n  </ul>\n</ul>\n",   /*17 ul */
/* 18 */   "<ul class='disc'>\n  <li>\n    \\2\n  </li>\n</ul>",   /*18 ul  */
/* 19 */   "",                              /*19 unordered list remove <\/ul>\n<ul>*/
/* 20 */   "",                              /*20 unordered list remove <\/ul>\n<ul>*/
/* 21 */   "",                              /*21 unordered list remove <\/ul>\n<ul>*/
/* 22 */   "",                              /*22 unordered list remove <\/ul>\n<ul>*/
/* 22a */  "#&nbsp;",              /*26 ordered list level 1: white space  */
/* 23 */   "<ol class='decimal'>\n  <ol class='alpha'>\n    <ol class='romanupper'>\n      <ol class='alphaupper'>\n        <li>\\2\n        </li>\n      </ol>\n    </ol>\n  </ol>\n</ol>",   /*23 ol */
/* 24 */   "<ol class='decimal'>\n  <ol class='alpha'>\n    <ol class='romanupper'>\n      <li>\n        \\2\n      </li>\n    </ol>\n  </ol>\n</ol>",   /*24 ol */
/* 25 */   "<ol class='decimal'>\n  <ol class='alpha'>\n    <li>\n      \\2\n    </li>\n  </ol>\n</ol>",   /*25 ol */
/* 26 */   "<ol class='decimal'>\n  <li>\n    \\2\n  </li>\n</ol>",   /*26 ol */
/* 27 */   "",                              /*27 ordered list remove <\/ol>\n<ol>*/
/* 28 */   "",                               /*28 ordered list remove <\/ol>\n<ol>*/
/* 29 */   "",                              /*29 ordered list remove <\/ol>\n<ol>*/
/* 30 */   "",                              /*30 ordered list remove <\/ol>\n<ol>*/
/* 31 */   "<dl>\n  <dt>\\2</dt><dd>\\3</dd>\n</dl>", /*31 definitions */
/* 32 */  "",              /* 32 multiple definitions */
// /* 33 */  "\\1<sup class='small' >\\2</sup> ", /* 33 superscript */
/* 36 */  "\\1&rarr;&nbsp;",// right arrow
/* 37 */  "\\1&larr;&nbsp;",// left arrow
/* 38 */  "\\1&harr;&nbsp;",// left right arrow
/* 39 */  "\\1&rArr;&nbsp;",// double right arrow
/* 40 */  "\\1&lArr;&nbsp;",// double left arrow
/* 41 */  "\\1&hArr;&nbsp;",// double left right arrow
/* 42 */  "&nbsp;&ndash; ", // &ndash;
/* 43 */  "&nbsp;&mdash; ", // &mdash;
/* 44 */  "<span title='female'>&#9792;</span>", // female
/* 45 */  "<span title='male'>&#9794;</span>", // male
/* last 1 */"<",  /* < sign */
/* last 2 */">",  /* > sign */
/* 34 */  "\\1&le;\\2",// less or equal
/* 35 */  "\\1&ge;\\2",// greater or equal
/* 48 */  "\\1&plusmn;\\2",//plus minus
/* 49 */  "\\1</th><th>\\2",//table head
/* 50 */  "\\1    </td>\n      \\2<td class='auto' >",//table data
/* 51 */  "'<h'.strlen('\\1').' class=\'sec'.strlen('\\1').'\' id=\'\\2\'>\\2</h'.strlen('\\1').'>'.putToArray('<a class=\'sec'.strlen('\\1').'\' href=\'#'.rawurlencode('\\2').'\'>\\2</a>',\$_SESSION[\"sections\"]).''",//sections levels
/* 52 */  "<p class='paragraph'>\n  \\1\n</p>\n",//paragraph
/* 53a*/  "&nbsp;",// non breaking space
// /* 53b*/  "\\1\\2~\\3\\4",// non breaking space in URL
/* 54 */  "'<span class='.str_replace(\" \",\"\",'\\1').'>\\1</span>'",// non breaking space
/* 55 */  "'<span class=\"overlib\" onmouseover=\"return overlib(\''.str_replace(\"'\",\"\'\",'\\2').'\', FGCOLOR, \'FFFFFF\' , OFFSETY, 20, CENTER );\" onmouseout=\"return nd();\">\\1</span>'",// non breaking space
/* 56 */  "Template:internal:hiddenMap",//
    );
    $this->text=stripslashes($text);
    $this->text=preg_replace( $searchpattern, $replacepattern,$this->text);
//      echo "tools:<br>";print_r($this->text);
    return $this->text;
  }//end function pattern2tag()


/**
 * remove some HTML syntax to wikipedia like ones
 *
 * @param string $text given HTML text
 * @return string
 * @internal currently not used htmlentities($_POST['textarea'],ENT_NOQUOTES,'UTF-8') is used instead
 */

    function tag2pattern($text)//special patterns:
    {
    $searchtags=array(
 /* 1 */  "/(<table)([^>]*)(>)/",
 /* 2 */  "/<\/table>/",
 /* 3 */  "/(<th)([^>]*)(>)/",
 /* 4 */  "/<\/th>/",
 /* 5 */  "/<tr>/",
 /* 6 */  "/<\/tr>/",
 /* 7 */  "/<td>/",
 /* 8 */  "/(<td)(.+?)(>)/",
 /* 9 */  "/<\/td>/",
 /* 10*/  "/(<!\-\-)(.+)(\-\->)/",/* comment */
 /* 11*/  "/(<\s*)([0-9].+\.|,[0-9]*)/",/* eg text: '<1.5 times' */
 /* 12*/  "/(>\s*)([0-9].+\.|,[0-9]*)/",/* eg text: '>1.5 times' */
 /* 13*/  "/(<b>)(.+)(<\/b>)/",
 /* 14*/  "/(<strong>)(.+?)(<\/strong>)/",
 /* 15*/  "/(<em>)(.+?)(<\/em>)/",
 /* 16*/  "/(<a)(\s+)(href=)('|\")(.+?)('|\")(\s*)(>)(.+?)(<\/a>)/",
/* 17*/  "/^(?>([=]+))([^=]+)(?>([=]+)) +$/m",
 // /* 18*/  "/<li>|</li>/",
    );
    $replacetags=array(
 /* 1 */  "{|\\2",/* <table> start */
 /* 2 */  "|}",/* </table> end */
 /* 3 */  "!\\2|",/* <th> start */
 /* 4 */  "",/* </th> end */
 /* 5 */  "|-",/* <tr> start */
 /* 6 */  "",/* </tr> end */
 /* 7 */  "|",/* <td> start */
 /* 8 */  "|\\2|",/* <td> start */
 /* 9 */  "",/* </td> end*/
 /* 10*/  "&lt;!--\\2--&gt;",/* comment */
 /* 11*/  "&lt;\\2", /* eg text: '<1.5 times' */
 /* 12*/  "&gt;\\2", /* eg text: '>1.5 times' */
 /* 13*/  "''\\2''",/* <b> */
 /* 14*/  "''\\2''",/* <strong> */
 /* 15*/  "'''\\2'''",/* <em> */
 /* 16*/  "[\\5 \\9]",/* <a> */
/* 17*/   "\1\2\3",/* correct sections */
  // /* 17*/  "",/* <a> */
    );
     // print_r($searchtags);
    $this->text['text']=preg_replace( $searchtags, $replacetags,$text['text']);
    if(!preg_match("/\\'/",$text['text']))
    {$this->text['text']=addslashes($this->text['text']);}
     // $this->text['text']=strip_tags($this->text['text'],'<b><br><div><em><hr><iframe><pre><span><strong><sub><sup>');
     // print_r($this->text['text']);
     // $text['text']=$this->text['text'];

  }//end function tag2pattern()

/**
 * small icon pencil
 *
 * @param string $php_vars php variables
 * @param string $file which file is used?
 * @param string $icon which icon
 * @return HTML <a>Pencil Icon</a>
 * @category edit
 */

  function pencil(
    $php_vars="",
    $file="popup.php",
    $html="title='Click to edit in a new window' ",
    $icon="b_edit.png"){
    /**
    * Extract current server path relatively
    */
    include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
    if(!empty($php_vars)){
    if(preg_match("#Firefox#i",$_SERVER['HTTP_USER_AGENT'])){
      return "\n  <span class='printinvisible' >\n    <a class='nothing' href='".$file."?".$php_vars."' target='_blank'>\n    <img  src='../structure/img/".$icon."' ".$html." align='middle' >\n    </a>\n  </span>";

    }# Firefox browser
    else{
      return "\n  <span class='printinvisible' >\n    <a class='nothing' onclick=\"window.open('".$file."?".$php_vars."', '','toolbar=no,status=no,menubar=no,scrollbars=yes,width=".$config['popup']['windowWidth'].",height=".$config['popup']['windowHeight']."');void(0);\">\n    <img  src='../structure/img/".$icon."' ".$html." align='middle' >\n    </a>\n  </span>\n";
      }
      #return "<span class='printinvisible' ><a class='nothing' href=\"javascript:window.open('".$file."?".$php_vars."', '','toolbar=no,status=no,menubar=no,scrollbars=yes,width=".$config['popup']['windowWidth'].",height=".$config['popup']['windowHeight']."');void(0);\">\n  <img  src='../structure/img/".$icon."' ".$html." align='middle' >\n</a>\n</span>";
    }
  }// end function pencil

/**
 * menu bar for popup.php
 *
 * If you want to change the menu with its inserted code (JavaScript) you can alter/add values
 * Just copy and modify a <samp><span>...</span></samp> tag
 * @param string $pushcontent  which text to insert
 * @param string $type  types are font, tools, list, table, timestamp, image, sections, openlayermap, wrapmodus
 * @param string $add  content to be added
 * @return HTML <samp><div class='menuicons'>Menu elements...</div></samp>
 * @category menu
 *
 */
# @annotation menu e.g. in popup.php
  function menubar($pushcontent='',$type='',$add=''){
  // usage: menubar(FormWhereTo,Type)
    /**
    * Extract current server path relatively
    */
  include(preg_replace('#(.+/)([^/]+/){1}[^/]+\.php#',"\\1\\3", __FILE__).'config.php');
  switch ($type) {
# @annotation menu: font
  case 'font':
  $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='bold type face' onclick=\"insert('\'\'','\'\'','$pushcontent','textarea');\" > <img src='../structure/img/tag_bold.png' width='22' height='22' alt='bold'> </span>
    <span style='text-align:center;' title='emphasised'  onclick=\"insert('\'\'\'','\'\'\'','$pushcontent','textarea');\"><img src='../structure/img/tag_i.png' width='22' height='22' alt='emphased' ></span>
    <span style='text-align:center;' title='bold and emphasised'  onclick=\"insert('\'\'\'\'\'','\'\'\'\'\'','$pushcontent','textarea');\" ><img src='../structure/img/tag_bold-em.png' width='22' height='22' ></span>
  </div>\n";
    break;
# @annotation menu: tools
  case 'tools':
  $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert line break' onclick=\"insert('<br>','','$pushcontent','textarea');\"  ><img src='../structure/img/tag_br_small.png' width='16' height='16' alt='line break'></span>
    <span style='text-align:center;' title='insert horizontal line' onclick=\"insert('\\n<hr>','\\n','$pushcontent','textarea');\"  ><img src='../structure/img/tag_hr.png' width='16' height='16' alt='line break'></span>
    <span style='text-align:center;' title='insert HTML comment' onclick=\"insert('<!-- ',' -->','$pushcontent','textarea');\"  ><img src='../structure/img/tag_comm.png' width='22' height='22' alt='comment'></span>
    <span style='text-align:center;' title='insert HTML tag' onclick=\"var HTMLtag = prompt('Insert an arbitrary HTML tag:','caption');insert('<'+HTMLtag+'>','</'+HTMLtag+'>','$pushcontent','textarea');\"  ><img src='../structure/img/tag_misc.png' width='22' height='22' alt='HTML tag'></span>
  </div>\n";
    break;
# @annotation menu: list
  case 'list':
  $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert unordered list' onclick=\"insert('\\n*','\\n','$pushcontent','textarea');\" >
    <img src='../structure/img/tag_ul.png' width='22' height='22' alt='unordered list'  align='middle' ></span>
    <span style='text-align:center;' title='insert ordered list' onclick=\"insert('\\n#','\\n','$pushcontent','textarea');\"  ><img src='../structure/img/tag_ol.png' width='22' height='22' alt='ordered list' align='middle' ></span>
    <span style='text-align:center;' title='insert definition list' onclick=\"insert('\\n;definition 1 : description 1...','\\n','$pushcontent','textarea');\" >
    <img src='../structure/img/b_definition.png'  alt='definition list' align='middle' ></span>
  </div>\n";
    break;
# @annotation menu: table
  case 'table':
    $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert a whole table' onclick=\"insert('\\n{| style=\'border:1px solid;\' class=\'sortable\' id=\'sortone\'\\n\\n|-\\n<!-- table head-->\\n!| Reference\\n!im|Taxon\\n!im90|table head as img + 90°\\n\\n<!-- table body -->\\n|-\\n|t.data1 row1\\n|t.data2 row1\\n|t.data3 row1\\n\\n|-\\n|t.data1 row2\\n|t.data2 row2\\n|t.data3 row2\\n\\n|-class=\'sortbottom\'\\n|row3||sum||12\\n','\\n|}','$pushcontent','textarea');\"  ><img src='../structure/img/tag_table.png' width='22' height='22' alt='table'></span>
    <span style='text-align:center;' title='insert whole table (systematic notes)' onclick=\"insert('\\n{| style=\'border:1px solid;\' class=\'sortable\' id=\'sortone\'\\n\\n|-\\n<!-- table head -->\\n!|Reference\\n!|Taxon\\n!im90|Head\\n!im90|Head: postoccipital plate\\n!im90|Head: postoccipital margin\\n!im90|Antenna: segments\\n!im90|Antenna: pedestal\\n!im90|Antenna: spur\\n!im90|Mentum: middle\\n!im90|Mentum: submental setation\\n!im90|Mentum: ventromental plates\\n!im90|Labrum: premandible\\n!im90|Labrum: seta S1\\n!im90|Labrum: pecten epipharyngis\\n!im90|Mandible (inner-apical-outer-surface)\\n!im90|Body: parapods\\n!im90|Body: size\\n!im90|Notes\\n!im90|Literature\\n\\n<!-- table body -->\\n|-\\n|<!-- Reference -->\\n|<!-- Taxon -->\\n|<!-- Head -->\\n|<!-- Head: postoccipital plate -->\\n|<!-- Head: postoccipital margin -->\\n|<!-- Antenna: segments -->\\n|<!-- Antenna: pedestal -->\\n|<!-- Antenna: spur -->\\n|<!-- Mentum: middle -->\\n|<!-- Mentum: submental setation -->\\n|<!-- Mentum: ventromental plates -->\\n|<!-- Labrum: premandible -->\\n|<!-- Labrum: seta S1 -->\\n|<!-- Labrum: pecten epipharyngis -->\\n|<!-- Mandible (inner-apical-outer-surface) -->\\n|<!-- Body: parapods -->\\n|<!-- Body: size -->\\n|<!-- Notes -->\\n|<!-- Literature -->\\n','\\n|}','$pushcontent','textarea');alert('Note that table data are sorted by their contents. Therefore proper results with:\\n  |6mm<!-- Body: size -->\\nNOT:\\n  |<!-- Body: size -->6mm');\"  ><img src='../structure/img/tag_table_syst.png' width='22' height='22' alt='table'></span>
    <span style='text-align:center;' title='insert table head' onclick=\"insert('\\n!|','\\n','$pushcontent','textarea');\"  ><img src='../structure/img/table_head.png' width='22' height='22' alt='table head'></span>
    <span style='text-align:center;' title='insert table row' onclick=\"insert('\\n|-\\n','','$pushcontent','textarea');\"  ><img src='../structure/img/tag_table_row.png' width='22' height='22' alt='table row'></span>
    <span style='text-align:center;' title='insert table data' onclick=\"insert('\\n|','\\n','$pushcontent','textarea');\"  ><img src='../structure/img/table_data.png' width='22' height='22' alt='table head'></span>
  </div>\n";
  $menubar.="<div class='menuicons'>
    <span style='text-align:center;' title='insert whole filterable table' onclick=\"insert('{| style=\'border:1px solid;\' id=\'sortTaxaOne\' class=\'table-autosort table-autofilter\'\\n<thead>\\n|-\\n!class=\'table-sortable:default\'|Reference\\n!im class=\'table-sortable:default\'|Taxon\\n!im90 class=\'table-sortable:numeric\'|seta S1\\n|-\\n!class=\'table-filterable\'|\\n!|<input name=\'filterTaxon\' size=\'8\' onkeyup=\'Table.filter(this,this)\'>\\n!|<input name=\'filterSet1\' size=\'3\' onkeyup=\'Table.filter(this,this)\'>\\n\\n</thead>\\n<tbody>\\n|-\\n|ref1<!--Reference-->\\n|T.gracilentus<!--Taxon-->\\n|1\\n\\n|-\\n|ref1<!--Reference-->\\n|T.gracilentus<!--Taxon-->\\n|2\\n\\n|-\\n|ref2<!--Reference-->\\n|T.austriacus<!--Taxon-->\\n|1\\n\\n|-\\n|ref3<!--Reference-->\\n|C.ambigua<!--Taxon-->\\n|1.5\\n</tbody>\\n\\n<tfoot>\\n<tr>\\n<td class=\'auto\' colspan=\'3\'>Some annotations </td>\\n</tr>\\n</tfoot>\\n\\n|}\\n\\n','','$pushcontent','textarea');\"  ><img src='../structure/img/tag_table_filter.png' width='22' height='22' alt='line break'></span>
   <span title='table wizard...' onclick=\"return overlib(' <form name=\'insertTable\'> rows x cols: <input name=\'tableNrows\' type=\'text\' size=\'2\' value=\'2\'> x <input name=\'tableNcols\'  type=\'text\' size=\'2\' value=\'2\' disabled=\'true\'><br> headers (one, two, ...):<br><textarea name=\'tableHeaders\' rows=\'5\' style=\'width:200px;\' onkeyup=\'document.insertTable.tableNcols.value = document.insertTable.tableHeaders.value.split(\&#034;,\&#034;).length;\'>Reference, Taxon</textarea><br> foot? <input name=\'tableFoot\' type=\'checkbox\'>&nbsp;&nbsp; auto sort? <input name=\'tableSorted\' type=\'checkbox\' value=\'auto\' onclick=\'tableFeatures[\&#034;sort\&#034;] = (this.checked)? this.value : \&#034;none\&#034;;\'><div style=\'border-top: 1px dotted gray;\'> filtering? <input name=\'tableFilterEnable\' type=\'checkbox\' onclick=\'for (var i = 0; i < document.insertTable.tableFilteredHow.length; i++) {document.insertTable.tableFilteredHow[i].disabled =  (this.checked ? false : true);if(this.checked){ if(document.insertTable.tableFilteredHow[i].checked) tableFeatures[\&#034;filter\&#034;] = document.insertTable.tableFilteredHow[i].value } else tableFeatures[\&#034;filter\&#034;] = \&#034;none\&#034;};\'><br> &nbsp;&nbsp;auto filter? <input name=\'tableFilteredHow\' type=\'radio\' value=\'auto\' disabled=\'true\' onclick=\'tableFeatures[\&#034;filter\&#034;] = this.value\'> or manual? <input name=\'tableFilteredHow\'  type=\'radio\' value=\'manual\' checked=\'true\' disabled=\'true\' onclick=\'tableFeatures[\&#034;filter\&#034;]  = this.value\'> </div> <div style=\'border-top: 1px dotted gray;\'>image headings? <input name=\'tableHeadImgEnable\' type=\'checkbox\' onclick=\'for (var i = 0; i < document.insertTable.tableHeadImgHow.length; i++) {document.insertTable.tableHeadImgHow[i].disabled =  (this.checked ? false : true);if(this.checked){ if(document.insertTable.tableHeadImgHow[i].checked) tableFeatures[\&#034;image\&#034;] = document.insertTable.tableHeadImgHow[i].value } else tableFeatures[\&#034;image\&#034;] = \&#034;none\&#034;};\'><br> &nbsp;&nbsp;auto? (".$config['img_text_angle']."°) <input name=\'tableHeadImgHow\' type=\'radio\' value=\'auto\' disabled=\'true\' onclick=\'tableFeatures[\&#034;image\&#034;] = this.value\'> or vertical (90°)? <input name=\'tableHeadImgHow\'  type=\'radio\' value=\'manual\' checked=\'true\' disabled=\'true\' onclick=\'tableFeatures[\&#034;image\&#034;]  = this.value\'></div> <div style=\'text-align:right;border-top:1px solid gray;\'><input value=\'insert\' type=\'button\' onclick=\'javascript:insert(tableCompose(document.insertTable.tableNrows.value, document.insertTable.tableNcols.value, document.insertTable.tableHeaders.value.replace(/[\\\\n\\\\r]+/ig,\&#034; \&#034;), document.insertTable.tableFoot.checked),\&#034;\&#034;,\&#034;$pushcontent\&#034;,\&#034;textarea\&#034;);\'></div> </form>',BELOW, CENTER, STICKY , CAPTION, 'Table Wizard' );\"  onmouseout=\"return nd();\"><img src='../structure/img/tag_table_wizard.png' width='22' height='22' alt='line break'></span>
  </div>";
    break;
# @annotation menu: timestamp
  case 'timestamp': //data.style.backgroundColor= (data.style.backgroundColor=='#900')?'#090':'#900';
    $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert actual time stamp' onclick=\"insert('('+Date()+')','','$pushcontent','textarea');\"  ><img src='../structure/img/time.png' width='16' height='16' alt='line break'></span>
  </div>\n";
    //\"alert(document.all.textarea.wrap);document.all.textarea.wrap=(document.all.textarea.wrap=='off')?'soft':'off';\"
    break;
# @annotation menu: image
  case 'image': //data.style.backgroundColor= (data.style.backgroundColor=='#900')?'#090':'#900';
    $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert image file' onclick=\"insert('<img width=70 title=\'Title of image\' src=\'../../".$config['dirSpec']."imagefile.jpg\'  align=\'middle\'>','','$pushcontent','textarea');\"  ><img src='../structure/img/image.png' width='22' height='22' alt='image' title='insert image'></span>
    <span style='text-align:center;' title='insert a text image' onclick=\"insert('<img title=\'imagetext default\'  src=\'image.php?text=imagetext default\'>','','$pushcontent','textarea');\"  ><img src='../structure/img/b_text-angle.png' width='22' height='22' alt='text image' title='insert text image'></span>
    <span style='text-align:center;' title='insert text image rotated' onclick=\"insert('<img title=\'imagetext rotated\'  src=\'image.php?text=imagetext rotated&amp;angle=90\'>','','$pushcontent','textarea');\"  ><img src='../structure/img/b_text-vert.png' width='22' height='22' alt='text image rotated' title='text image rotated'></span>
    <span style='text-align:center;' title='insert overlib text or image' onclick=\"insert('{overlib|text ...','|overlib-content}','$pushcontent','textarea');\"  ><img src='../structure/img/b_overlib.png' width='30' height='30' alt='overlib text or image' title='overlib text or image'></span>
  </div>\n";
    //\"alert(document.all.textarea.wrap);document.all.textarea.wrap=(document.all.textarea.wrap=='off')?'soft':'off';\"
    break;
# @annotation menu: sections
  case 'sections': //data.style.backgroundColor= (data.style.backgroundColor=='#900')?'#090':'#900';
    $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert section level 1' onclick=\"insert('\\n=section level 1=\\n','','$pushcontent','textarea');\"  ><img src='../structure/img/section1.png' width='22' height='22' alt='image' title='section level 1'></span>
    <span style='text-align:center;' title='insert section level 2'  onclick=\"insert('\\n==section level 2==\\n','','$pushcontent','textarea');\"  ><img src='../structure/img/section2.png' width='22' height='22' alt='image' title='section level 2'></span>
    <span style='text-align:center;' title='insert section level 3'  onclick=\"insert('\\n===section level 3===\\n','','$pushcontent','textarea');\"  ><img src='../structure/img/section3.png' width='22' height='22' alt='image' title='section level 3'></span>
    <span style='text-align:center;' title='insert section level 4'  onclick=\"insert('\\n====section level 4====\\n','','$pushcontent','textarea');\"  ><img src='../structure/img/section4.png' width='22' height='22' alt='image' title='section level 4'></span>
  </div>\n";
    break;
# @annotation menu: openlayermap
  case 'openlayermap': //data.style.backgroundColor= (data.style.backgroundColor=='#900')?'#090':'#900';
    $menubar="
  <div class='menuicons'>
    <span style='text-align:center;' title='insert OpenLayers map' onclick=\"insert('\\n{{openLayer|1|\\n(30.2837\$29.3494\$Title\$possible description)','\\n}}','$pushcontent','textarea');alert('Works only with internet access.\\ndecimal:\\n  (63.3434798\$60\$title\$description)\\ndegree:\\n  (36°23\'34\'\'$30°0\'0\'\'\$title\$description)\\na point:\\n  (30.2837\$29.3494\$title\$description\$o)\\n\\nNote: you can also comment this out to keep data stored but hide the map.');\"  ><img src='../structure/img/b_world-mini.png'   alt='openLayer map' title='openLayer Map'></span>
  </div>\n";
    break;
# @annotation menu: wrapmodus
  case 'wrapmodus': //data.style.backgroundColor= (data.style.backgroundColor=='#900')?'#090':'#900';
    $add.=empty($add) ? '':"<br>" ;
    $menubar="
  <div class='menuicons'>
    ".$add."
    <span style='text-align:center;' title='wrap text after save' >yes:&nbsp;<input type='radio' name='wrap' value='on' ".(isset($_POST['wrap'])?($_POST['wrap']=="on"?"checked='true'":""):"checked='true'").">&nbsp;no:&nbsp;<input type='radio' name='wrap' value='off' ".(isset($_POST['wrap'])? ($_POST['wrap']=="on"?"":"checked='true'"):"").">
  </span>
  </div>\n";
    break;
  case 'languages':
  if(isset($config['languages']['keys']))
    foreach ($config['languages']['keys'] as $key => $language) {
      $selection[$key] = $language;
    }// end foreach
    $selection = self::select($selection, "language", isset($_POST['language'])? $_POST['language'] :$config['languages']['default'] , " title='choose language for special characters' ", 'value');
    $menubar="
  <div class='menuicons'>
    ".$selection."
  </div>\n";
    break;
    }// end case
    return $menubar;
  }// end function menubar

/**
 * Searches a string in online databases
 *
 * @param string $strSearch (composed) string to be searched, e.g.
 * 'Genus+species'. Is splitted sometimes.
 * @param mixed  $category  is one of 'ITIS', 'GoogleImgage', 'GoogleScholar', 'Scirius', 'ScienceDirect', 'AskImages', 'Jstor', 'FaunaEuropea'
 * @todo use POST methods
 * @return HTML
 */

  function searchStringOnline($strSearch, $category='', $isEscaped=false)  {
    $strSearchSplit = explode("+",$strSearch); // split into pieces by '+'
    switch ($category) {
    case "Species2000": # @annotation searchStringOnline: Species2000
      $strUrl = "http://www.catalogueoflife.org/search_results.php?search_string=".$strSearch."&amp;match_whole_words=on";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Species 2000 Checklist' src='../structure/img/b_species2000.png' height=16  border=0></a>";
      break;
    case "ITIS":    # @annotation searchStringOnline: ITIS
      $strUrl = "http://www.itis.gov/servlet/SingleRpt/SingleRpt?search_topic=Scientific_Name&amp;search_kingdom=Animal&amp;search_span=containing&amp;search_value=".$strSearch."&amp;Go=Search&amp;categories=All&amp;source=html&amp;search_credRating=All";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: ITIS Checklist (Integrated Taxonomic Information System)' src='../structure/img/b_itis.png' height=16  border=0></a>";
      break;
    case "GoogleImgage":    # @annotation searchStringOnline: GoogleImgage
      $strUrl =  "http://images.google.com/images?svnum=100&amp;newwindow=1&amp;q=".$strSearch."&amp;ie=UTF-8&amp;oe=UTF-8&amp;hl=en&amp;btnG=Search";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Image Search Google' src='../structure/img/google_imagesearch.gif' height=16  border=0></a>";
      break;
    case "GoogleScholar":    # @annotation searchStringOnline: GoogleScholar
      $strUrl =  "http://scholar.google.com/scholar?q=".$strSearch."&amp;ie=UTF-8&amp;oe=UTF-8&amp;hl=en&amp;btnG=Search";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Scholar Search Google' src='../structure/img/google_scholarsearch.gif' height=16  border=0></a>";
      break;
    case "Scirius":    # @annotation searchStringOnline: Scirius
      $strUrl =  "http://www.scirus.com/srsapp/search?q=".$strSearch."&amp;ds=jnl&amp;ds=web&amp;g=s&amp;t=all";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Journal Search Scirius' src='../structure/img/scirus_search.gif' height=16  border=0></a>";
      break;
    case "AskImages":    # @annotation searchStringOnline: AskImages
      $strUrl =  "http://images.ask.com/pictures?q=".$strSearch;
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Image Search Ask' src='../structure/img/ask_search.gif' height=16  border=0></a>";
      break;
    case "ScienceDirect":    # @annotation searchStringOnline: ScienceDirect
      $strUrl =  "http://www.sciencedirect.com/science?_ob=QuickSearchURL&amp;_method=submitForm&amp;_acct=C000054205&amp;md5=d7f83b6e67cdd758968256bfc76dc3c3&amp;qs_tak=".$strSearch."&amp;qs_author=&amp;qs_title=&amp;qs_vol=&amp;qs_issue=&amp;qs_pages=&amp;x=0&amp;y=0";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Journal Search Science Direct' src='../structure/img/sciencedirect_search.gif' height=16  border=0></a>";
      break;
    case "Jstor":    # @annotation searchStringOnline: Jstor
      $strUrl =  "http://www.jstor.org/action/doBasicSearch?Query=".$strSearch."&amp;wc=on&amp;dc=All+Disciplines";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Journal Search JStor' src='../structure/img/jstor_search.gif' height=16  border=0></a>";
      break;
    case "FaunaEuropea":    # @annotation searchStringOnline: FaunaEuropea
      $strUrl =  "http://www.faunaeur.org/index.php?genus_search_mode=is&amp;genus=".$strSearchSplit[0]."&amp;species_search_mode=is&amp;species=".$strSearchSplit[1]."&amp;submit=Search&amp;show_what=search+results&amp;species_or_higher_taxa=species";
      $strUrl = "<a href='".$strUrl."' target='_blank'><img align='middle' title='New Window: Fauna Europaea Checklist' src='../structure/img/b_europa.png' height=16  border=0></a>";
      break;
    default:# @annotation searchStringOnline: default IXQUICK
      $strUrl = "http://eu.ixquick.com/do/metasearch.pl?cat=web&amp;cmd=process_search&amp;language=english&amp;engine0=v1all&amp;ff=&amp;query=".$strSearch;
      $strUrl = "<a href='".$strUrl."' target='_blank' title='New Window: Ixquick Meta Search' ><img src='../structure/img/ixquick_search.gif' height=16 border=0></a>";
    break;
    }// end switch
    if($isEscaped)
      $strUrl = str_replace("'","\'",$strUrl);

    return $strUrl." ";
  }// end searchStringOnline
}//end class html


/**
 * Class for text images
 *
 * @author Andreas Plank <andreas.plank@web.de>
 * @version 2009-02-24 21:24:33
 * @license Noncommercial-Share Alike 3.0 Unported
 * @category program tools
 * @package chirokey_larvae
 * @subpackage program
 */

class  pictures {
  /*
  public - überall zugänglich, lesen, verändern
  protected - innerhalb und in abgeleiteter Klasse
  private - NUR innerhalb Klasse
  */
  public $text;
  public $background_color='';
  public $angle=50;

  function errors($error_message){
    /**
    * Extract current server path relatively
    */
    include(preg_replace('#(.+/)([^/]+/){2}[^/]+\.php#',"\\1\\3", __FILE__).'structure/config.php');

    $error_message=wordwrap($error_message, 30);
    $text_mesure=imagettfbbox($config['fontsize'],0,$config['fontpath'],$error_message);
    $height= abs($text_mesure[1])+abs($text_mesure[7]);
    $width = abs($text_mesure[0])+abs($text_mesure[2]);
    $text_at_x=0;
    $text_at_y=abs($text_mesure[7]);

    $im = ImageCreate($width, $height);
    $background_color = ImageColorAllocate ($im, 255, 255, 25);
    $text_color = ImageColorAllocate ($im, 233, 14, 91);

    imagettftext($im, $config['fontsize'], 0, $text_at_x, $text_at_y, $text_color, $config['fontpath'], $error_message);
    imagepng ($im);
    imagedestroy($im);
    exit;// exit skript here
  }# end function picture_errors()

    /**
     * return width and height, offset [left, top] of a ttf character
     *
     * adapted from: http://de2.php.net/manual/de/function.imagettfbbox.php#53736
     * @param string $font : the font file
     * @param string $text : the character
     * @param int $size : the font size
     * @param int $angle : the angle
     * @access private
     * @return array  of the width and height, left and top.
     */
   function getCharacterSize($font, $text, $size, $angle)
   {
         // Get the boundingbox from imagettfbbox(), which is correct when angle is 0
         $bbox = imagettfbbox($size, 0, $font, $text);

         // Rotate the boundingbox
         $angle = deg2rad($angle/ 180);
         for ($i=0; $i<4; $i++)
         {
            $x = $bbox[$i * 2];
            $y = $bbox[$i * 2 + 1];
            $bbox[$i * 2] = cos($angle) * $x - sin($angle) * $y;  // X
            $bbox[$i * 2 + 1] = sin($angle) * $x + cos($angle) * $y ; // Y
         }
         // Variables which tells the correct width and height
       $bbox["left"] = 0- min($bbox[0],$bbox[2],$bbox[4],$bbox[6]);
       $bbox["top"] = 0-2* min($bbox[1],$bbox[3],$bbox[5],$bbox[7])-1;
       $bbox["width"] = max($bbox[0],$bbox[2],$bbox[4],$bbox[6]) -  min($bbox[0],$bbox[2],$bbox[4],$bbox[6]);
       $bbox["height"] = max($bbox[1],$bbox[3],$bbox[5],$bbox[7]) - min($bbox[1],$bbox[3],$bbox[5],$bbox[7]);
    # @annotation wrapped Text und normaler TExt stimmt noch nicht
       return $bbox;
   }# end function getCharacterSize()


    /**
     * writes text in table's headings rotated as image
     *
     * To ensure characters being transmitted base64_encode() is used instead of only htmlentities().
     * Note IE has some problems with transparency PNG. Possibly depending on PHP version the
     * boundingbox isn't fetched properly and width or height has to be adjusted then in imagettftext().
     * @param string $text : the text
     * @param int $angle : the angle
     * @return image.png
     */
  function write_text_picture($text,$angle)
  {
    /**
    * Extract current server path relatively
    */
    include(preg_replace('#(.+/)([^/]+/){2}[^/]+\.php#',"\\1\\3", __FILE__).'structure/config.php');

    if(empty($text))
    {/*if empty reference text -> blinking Text*/
      self::picture_errors('Can not create a text picture. There is no text.');/* stop execution */
    }
    else
    {/* create a referenced picture */
    //mesure text: array<-imagettfbbox ( int size, int angle, string fontfile, string text )
      $text=wordwrap($text,20);
   // Set some test variables
    $font = $config['fontpath'];
    $size = $config['fontsize'];
  // $bbox=getCharacterSize($font,$text,$size,$angle);
    $bbox_temp=self::getCharacterSize($font,$text,$size,$angle);

    $bb = imagettfbbox($size, $angle, $font, $text);
    /*
    0  lower left corner,  X position
    2  lower right corner, X position
    4  upper right corner, X position
    6  upper left corner,  X position
    1  lower left corner,  Y position
    3  lower right corner, Y position
    5  upper right corner, Y position
    7  upper left corner,  Y position
    */
    $bbox_temp['width'] = abs(
      max($bb[0],$bb[2],$bb[4],$bb[6]) -
      min($bb[0],$bb[2],$bb[4],$bb[6])
      );

    $bbox_temp['height'] =
      max($bb[1],$bb[3],$bb[5],$bb[7])-
      min($bb[1],$bb[3],$bb[5],$bb[7])
      ;
#333333
  // create image
    $im = imagecreatetruecolor($bbox_temp['width'], $bbox_temp['height']);
  // background color
    $background_color = hexdec("C0C0C0");
    imagefill($im, 0, 0, $background_color);
  // backgroundColor -> transparency
    imagecolortransparent($im,$background_color);
  // $text_color = ImageColorAllocate($im, 0, 0, 0);
    $text_color = hexdec("000000");
    $bbox_temp['attheleft']=abs($bb[6]);
    $bbox_temp['atthetop']=abs($bb[5])+1;
  // $bbox_temp['atthetop']=98;
  // resource $image , float $size , float $angle , int $x , int $y , int $color , string $fontfile , string $text
   imagettftext($im,   /*resource*/
                $size, /*size*/
                $angle, /*angle*/
                abs($bb[6])-2,           /*x*/
                abs($bb[5])-1,#+abs($bb[7]),$bbox_temp['height']),#$bbox_temp['height'],#abs($bb[5])+abs($bb[7]), /*y*/
                $text_color,
                $font,
                $text
                );
   // Show the image
   imagepng($im);
    // imagegif($im);

    }

  }// end function write_text_picture

}// end class pictures()

//unwraped
// -2   11
// 5  -311
// -8  -312
// -14  11

// wraped
// 39  12
// 42  -117
// -11 -118
// -14 11

    /**
     * write taxonomic levels
     *
     * @param string $level : genus/subfamily
     * @param int $name : the genus' name
     */

  function info_level($level="", $name) {
    $content.="\n<!-- taxonomic level -->\n<div id='taxonomicLevel' style=\"background-color : #DDDDDD; padding:0.5ex; position : fixed; top : 0px;right:1px; z-index : 10;\">".$level.": ".$name."</div>\n";
    return $content;
  }

  function info_start($title="Info", $css=FALSE) {
   $img = $css=='error' ? "<img src='../structure/img/s_warn.png' width='16' height='16' border='0'>" : '';
   $css = $css ? " class='".$css."' " : "";
   $info = "<fieldset ".$css.">\n  <legend ".$css.">".$title."</legend>\n";
   return $info.$img;
  }

  /**
   * html footer
   */
  function info_end() {
    return "\n</fieldset>\n";
  }

  /**
    *  for array_walk()
    */
  function user_htmlentities(&$item){
    $item =htmlentities($item,ENT_QUOTES,'UTF-8');
  }//end funktionsname

?>
